import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.Socket;

public class ReflectorPacket {
	private static final int HEADER_LENGTH = 4;
	private static final int COMMAND_OFFSET = 0;
	private static final int CHANNEL_OFFSET = 1;
	private static final int ID_OFFSET = 2;
	private static final int TARGET_COUNT_OFFSET = 3;

	private InetAddress udpAddress;
	private int udpPort;
	private byte[] buffer;
	private Socket socket;

	private String toHex(byte[] buf) {
		if (buf == null || buf.length == 0)
			return "";
		StringBuffer strbuf = new StringBuffer();
		for (int i = 0; i < buf.length; i++) {
			int c = buf[i];
			c = (c < 0) ? c + 0x100 : c;
			String tmpstr = "00" + Integer.toString(c, 16);
			strbuf.append(":" + tmpstr.substring(tmpstr.length() - 2));
		}
		return new String(strbuf);
	}

	public String toString() {
		return "ReflectorPacket[" + socket + ":" + udpAddress + ":"
			+ udpPort + "]" + toHex(buffer);
	}

	public ReflectorPacket(DatagramPacket packet) {
		this.buffer = new byte[packet.getLength()];
		System.arraycopy(packet.getData(), 0, this.buffer, 0,
			this.buffer.length);
		this.udpAddress = packet.getAddress();
		this.udpPort = packet.getPort();
		this.socket = null;
	}

	public ReflectorPacket(Socket sock, byte[] buf, int length) {
		this.buffer = new byte[length];
		System.arraycopy(buf, 0, this.buffer, 0, this.buffer.length);
		this.udpAddress = null;
		this.udpPort = -1;
		this.socket = sock;
	}

	public ReflectorPacket(byte[] buf, int length) {
		this.buffer = new byte[length];
		System.arraycopy(buf, 0, this.buffer, 0, this.buffer.length);
		this.udpAddress = null;
		this.udpPort = -1;
		this.socket = null;
	}

	public ReflectorPacket() {
		buffer = new byte[ID_OFFSET + 1];
		this.udpAddress = null;
		this.udpPort = -1;
		this.socket = null;
	}

	public InetAddress getUDPAddress() {
		return udpAddress;
	}

	public int getUDPPort() {
		return udpPort;
	}

	public Socket getSocket() {
		return socket;
	}

	public byte[] getBuffer() {
		return buffer;
	}

	public int getCommand() {
		int i = (int) buffer[COMMAND_OFFSET];
		i = (i < 0) ? i + 0x100 : i;
		return i;
	}

	public synchronized void setCommand(int command) {
		buffer[COMMAND_OFFSET] = (byte) command;
	}

	public int getChannel() {
		int i = (int) buffer[CHANNEL_OFFSET];
		i = (i < 0) ? i + 0x100 : i;
		return i;
	}

	public synchronized void setChannel(int channel) {
		buffer[CHANNEL_OFFSET] = (byte) channel;
	}

	public int getId() {
		int i = (int) buffer[ID_OFFSET];
		i = (i < 0) ? i + 0x100 : i;
		return i;
	}

	public synchronized void setId(int id) {
		if (buffer.length < ID_OFFSET + 1) {
			byte[] newbuf = new byte[ID_OFFSET + 1];
			System.arraycopy(buffer, 0, newbuf, 0, buffer.length);
			buffer = newbuf;
		}
		buffer[ID_OFFSET] = (byte) id;
	}

	public byte[] getTargets() {
		int count = (int) buffer[TARGET_COUNT_OFFSET];
		count = (count < 0) ? count + 0x100 : count;
		byte[] targets = new byte[count];
		System.arraycopy(buffer, HEADER_LENGTH, targets, 0, targets.length);
		return targets;
	}

	public synchronized void setTargets(byte[] targets) {
		int count;
		if (buffer.length <= TARGET_COUNT_OFFSET) {
			count = buffer.length - HEADER_LENGTH;
		} else {
			count = (int) buffer[TARGET_COUNT_OFFSET];
			count = (count < 0) ? count + 0x100 : count;
		}
		byte[] newbuf = new byte[buffer.length - count + targets.length];
		System.arraycopy(buffer, 0, newbuf, 0, TARGET_COUNT_OFFSET);
		newbuf[TARGET_COUNT_OFFSET] = (byte) targets.length;
		System.arraycopy(targets, 0, newbuf, HEADER_LENGTH, targets.length);
		System.arraycopy(buffer, HEADER_LENGTH + count,
			newbuf, HEADER_LENGTH + targets.length,
			buffer.length - count - HEADER_LENGTH);
		buffer = newbuf;
	}

	public byte[] getData() {
		if (buffer.length <= HEADER_LENGTH) {
			return null;
		}
		int count = (int) buffer[TARGET_COUNT_OFFSET];
		count = (count < 0) ? count + 0x100 : count;
		byte[] data = new byte[buffer.length - count - HEADER_LENGTH];
		System.arraycopy(buffer, count + HEADER_LENGTH, data, 0, data.length);
		return data;
	}

	public synchronized void setData(byte[] data) {
		if (data == null)
			return;
		int count = (int) buffer[TARGET_COUNT_OFFSET];
		count = (count < 0) ? count + 0x100 : count;
		byte[] newbuf = new byte[HEADER_LENGTH + count + data.length];
		System.arraycopy(buffer, 0, newbuf, 0, HEADER_LENGTH + count);
		System.arraycopy(data, 0, newbuf, HEADER_LENGTH + count, data.length);
		buffer = newbuf;
	}
}
